#!/usr/bin/python
# *-* coding: utf8 -*-
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

# PIE :
#	author : mmx900(http://m9.pe.kr, NO MAIL)
# 	status : lv 1 exp 1

import gtk, mgtk
from DocumentManager import *
from Window import *
from LineMover import *
from EnvironmentManager import *

class PIE:
	#객체 변수들
	window = '' #Main Window
	env = '' #Environment Manager

	def delete_event(self, widget, data=None):
		gtk.main_quit()
		return gtk.FALSE

	def __init__(self):
		self.window = Window()
		self.env = EnvironmentManager()
		self.window.set_title(self.env.get_title())
		self.docManager = DocumentManager(self)
		self.window.set_document(self.docManager)
		self.eventConnection()

	def eventConnection(self):
		self.window.connect("event", self.modifyChecker)
		#self.docManager.connect("event", self.modifyChecker)
		self.window.connect("delete_event", self.delete_event)
		self.window.connect("key-press-event", self.keyEventConnection)

		self.window.menu_items['new'].connect("activate", lambda w: self.fileNew())
		self.window.menu_items['open'].connect("activate", self.fileSelection, "open")
		self.window.menu_items['close'].connect("activate", self.fileClose)
		self.window.menu_items['save'].connect("activate", self.fileSelection, "save")
		self.window.menu_items['save_as'].connect("activate", self.fileSelection, "save_as")
		self.window.menu_items['quit'].connect("activate", self.delete_event)

		self.window.menu_items['line_move'].connect("activate", self.lineMove)

		self.window.toolbar_buttons['new'].connect("clicked", lambda w: self.fileNew())
		self.window.toolbar_buttons['open'].connect("clicked", self.fileSelection, "open")
		self.window.toolbar_buttons['close'].connect("clicked", self.fileClose)
		self.window.toolbar_buttons['save'].connect("clicked", self.fileSelection, "save")
		self.window.toolbar_buttons['save_as'].connect("clicked", self.fileSelection, "save_as")
		self.window.toolbar_buttons['quit'].connect("clicked", self.delete_event)

	def modifyChecker(self, widget, event):
		message = ""
		message += "Line : " + self.docManager.getDocument().get_current_line_number()
		message += " / "
		message += self.docManager.getDocument().get_total_line_count()
		message += "     "
		message += "Char : " + self.docManager.getDocument().get_current_char_number()
		message += " / "
		message += self.docManager.getDocument().get_total_char_count_at_current_line()
		message += " / "
		message += self.docManager.getDocument().get_total_char_count()
		message += "     "
		if (self.docManager.document.get_modified()):
			message += "Modified"
		else:
			message += "Original"
		self.window.push_message(message)
	
	def keyEventConnection(self, widget, event):
		# event.state : ctrl = 4, alt = 8, shift = 1
		# event.keyval : spacebar = 32

		#print event.state
		#print event.keyval
		#print event.string
		
		if (self.env.get_key_style() == 'WIN'):
			# CTRL(4) + n(110), N(2, 78)
			if ((event.state == 4 and event.keyval == 110) or
				(event.state == 6 and event.keyval == 78)): self.fileNew()
			# CTRL + s(115), S(2, 83)
			if ((event.state == 4 and event.keyval == 115) or
				(event.state == 6 and event.keyval == 83)): self.fileSelection(widget, 'save')
			# CTRL + o(111), O(2,79)
			if ((event.state == 4 and event.keyval == 111) or
				(event.state == 6 and event.keyval == 79)): self.fileSelection(widget, 'open')
			# CTRL + q(113), Q(2,81)
			if ((event.state == 4 and event.keyval == 113) or
				(event.state == 6 and event.keyval == 81)): self.delete_event(widget)
			# CTRL + g(103), G(2, 71)
			if ((event.state == 4 and event.keyval == 103) or
				(event.state == 6 and event.keyval == 71)): self.lineMove(widget)
			# CTRL + f(102), F(2,70)
			#if ((event.state == 4 and event.keyval == 102) or
			#(event.state == 6 and event.keyval == 70)): self.delete_event(widget)
			# CTRL + r(114), R(2,82)
			#if ((event.state == 4 and event.keyval == 114) or
			#(event.state == 6 and event.keyval == 82)): self.delete_event(widget)
		# self.docManager.repaintCurrentLine()


	def fileSelection(self,widget,arg1):
		if (self.docManager.getDocument().getFileName() == ''):
			self.filew = gtk.FileSelection("File selection")
			self.filew.connect("destroy", lambda w: self.filew.destroy())
			if (arg1 == "open"):
				self.filew.ok_button.connect("clicked", lambda w: self.fileOpen())
			elif (arg1 == "save"):
				self.filew.ok_button.connect("clicked", lambda w: self.fileSave())
			elif (arg1 == "save_as"):
				self.filew.ok_button.connect("clicked", lambda w: self.fileSave())
			self.filew.cancel_button.connect("clicked",lambda w: self.filew.destroy())
			self.filew.show()
		else:
			if ((arg1 == "open") or (arg1 == "save_as")):
				self.filew = gtk.FileSelection("File selection")
				self.filew.connect("destroy", lambda w: self.filew.destroy())
				if (arg1 == "open"):
					self.filew.ok_button.connect("clicked", lambda w: self.fileOpen())
				elif (arg1 == "save_as"):
					self.filew.set_filename(self.docManager.getFileName())
					self.filew.ok_button.connect("clicked", lambda w: self.fileSave())
				self.filew.cancel_button.connect("clicked",lambda w: self.filew.destroy())
				#self.filew.set_filename("penguin.png")
				self.filew.show()
			else:
				self.fileModify()

	def lineMove(self,widget):
		lm = LineMover(self.docManager.getDocument())

	def fileNew(self):
		self.docManager.newDoc()

	def fileClose(self,widget):
		self.docManager.closeDoc()

	def fileOpen(self):
		self.docManager.openDoc(self.filew.get_filename())
		self.window.set_title(self.env.get_title() + ': ' + self.filew.get_filename())
		self.filew.destroy()
		del(self.filew)

	def fileModify(self):
		self.docManager.docSaveUpdate()

	def fileSave(self):
		self.docManager.docSaveNew(self.filew.get_filename())
		self.filew.destroy()
		del(self.filew)

def main():
    gtk.main()

if __name__ == "__main__":
    hello = PIE()
    main()
